"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Logger = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
class Logger {
    constructor() {
        this.LOG_TYPE = {
            INFO: 'info',
            DEBUG: 'debug',
            ERROR: 'error',
            WARN: 'warn'
        };
        this._dateString = ({ time = true } = {}) => {
            const d = new Date();
            const year = d.getFullYear();
            const month = (`0${(d.getMonth() + 1)}`).slice(-2);
            const date = (`0${d.getDate()}`).slice(-2);
            const hours = (`0${d.getHours()}`).slice(-2);
            const minutes = (`0${d.getMinutes()}`).slice(-2);
            const seconds = (`0${d.getSeconds()}`).slice(-2);
            if (time)
                return `${year}-${month}-${date} ${hours}:${minutes}:${seconds}`;
            return `${year}-${month}-${date}`;
        };
        this._createFolder = () => {
            try {
                fs_1.default.accessSync(this.LOG_FOLDER, fs_1.default.constants.F_OK);
            }
            catch (err) {
                fs_1.default.mkdirSync(this.LOG_FOLDER, {
                    recursive: true
                });
            }
            return this;
        };
        this._createLogFiles = ({ type = 'error', message = '', stack = [] }) => {
            this._createFolder();
            const LOG_NAME = `${this.LOG_FOLDER}/${this.LOG_NAME}`;
            let data = '';
            if (type === 'error') {
                const stackTrace = stack.map((data, index) => `#${index} ${data.trim()}\n`);
                stackTrace.shift();
                data = `${this._dateString()} ${type}: ${message} \nStack trace: \n${stackTrace.join("")}`;
            }
            else
                data = `${this._dateString()} ${type}: ${message}`;
            try {
                if (!fs_1.default.existsSync(LOG_NAME))
                    fs_1.default.writeFileSync(LOG_NAME, data);
                else {
                    const old = fs_1.default.readFileSync(LOG_NAME, 'utf8');
                    const newData = old ? `${old}\n${data}` : data;
                    fs_1.default.writeFileSync(LOG_NAME, newData);
                }
            }
            catch (err) {
                console.log("can't push file logs");
            }
            return this;
        };
        this._findStack = (err) => {
            try {
                if (err.stack == null)
                    return [];
                const stackData = err.stack.split("\n");
                const stack = stackData !== null && stackData !== void 0 ? stackData : [];
                return stack;
            }
            catch (err) {
                return [];
            }
        };
        this.LOG_NAME = `${this._dateString({ time: false })}.log`;
        this.LOG_FOLDER = path_1.default.resolve("logs");
    }
    createLogger(name = 'logs') {
        this.LOG_FOLDER = path_1.default.resolve(name);
        return this;
    }
    readLogs(name = null) {
        const file = name ? `${this.LOG_FOLDER}/${name}` : `${this.LOG_FOLDER}/${this.LOG_NAME}`;
        const rawLogs = fs_1.default.readFileSync(file, 'utf8');
        const logs = rawLogs === null || rawLogs === void 0 ? void 0 : rawLogs.split('\n');
        return logs !== null && logs !== void 0 ? logs : [];
    }
    readFolder(name = null) {
        var _a;
        try {
            const file = name ? `${name}` : `${this.LOG_FOLDER}`;
            return (_a = fs_1.default.readdirSync(file)) !== null && _a !== void 0 ? _a : [];
        }
        catch (err) {
            console.log(err);
        }
    }
    error(err) {
        const stack = this._findStack(err);
        this._createLogFiles({
            type: this.LOG_TYPE.ERROR,
            message: err.message || 'unknow message',
            stack
        });
        return;
    }
    info(message) {
        this._createLogFiles({
            type: this.LOG_TYPE.INFO,
            message,
            stack: []
        });
        return;
    }
    warn(message) {
        this._createLogFiles({
            type: this.LOG_TYPE.WARN,
            message,
            stack: []
        });
        return;
    }
    debug(message) {
        this._createLogFiles({
            type: this.LOG_TYPE.DEBUG,
            message,
            stack: []
        });
        return;
    }
}
exports.Logger = Logger;
