"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StressTest = exports.Test = void 0;
const _1 = require(".");
class Test {
    describe(description) {
        this.description = description;
        return this;
    }
    name(description) {
        this.description = description;
        return this;
    }
    expect(callback) {
        this.callback = callback;
        return this;
    }
    equal(toBeValue) {
        return __awaiter(this, void 0, void 0, function* () {
            this.toBe(toBeValue);
        });
    }
    toBe(toBeValue) {
        return __awaiter(this, void 0, void 0, function* () {
            const startTime = process.hrtime();
            const diffTime = (hrtime) => {
                if (hrtime == null)
                    return;
                const [start, end] = process.hrtime(hrtime);
                return (start + (end / 1e9)).toFixed(4);
            };
            try {
                const result = yield this.callback();
                if (!(0, _1.deepEqual)(result, toBeValue))
                    throw new Error('value not match');
                console.log(`\x1b[42m PASSED \x1b[0m\x1b[34m (${this.description}) =>\x1b[0m \x1b[33mexecute time\x1b[0m : \x1b[32m${diffTime(startTime)}s\x1b[0m\n`);
            }
            catch (err) {
                console.log(`\x1b[41m FAILED \x1b[0m\x1b[34m (${this.description}) =>\x1b[0m \x1b[33mexecute time\x1b[0m : \x1b[32m${diffTime(startTime)}s\x1b[0m\n`);
            }
        });
    }
}
exports.Test = Test;
class StressTest {
    constructor() {
        this.EACH = 1;
    }
    each(each) {
        this.EACH = each;
        return this;
    }
    test(promise) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const promises = [...Array(this.EACH).keys()].map(_ => promise);
            const startTime = process.hrtime();
            const diffTime = (hrtime) => {
                if (hrtime == null)
                    return;
                const [start, end] = process.hrtime(hrtime);
                return (start + (end / 1e9)).toFixed(4);
            };
            const res = yield Promise.allSettled(promises);
            let reportsSuccess = 0;
            let reportsErrors = 0;
            let errors = [];
            for (const r of res) {
                if (r.status === 'fulfilled') {
                    reportsSuccess++;
                    continue;
                }
                errors.push((_c = (_b = (_a = r.reason.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.message) !== null && _c !== void 0 ? _c : (_d = r.reason) === null || _d === void 0 ? void 0 : _d.message);
                reportsErrors++;
            }
            return {
                successes: reportsSuccess,
                errors: reportsErrors,
                message: Object.entries(errors.reduce((r, c) => (r[c] = (r[c] || 0) + 1, r), {})).map(([k, v]) => v == 1 ? k : `(${v})` + ' ' + k),
                timer: `${diffTime(startTime)} s`
            };
        });
    }
}
exports.StressTest = StressTest;
